// XASSERT.H: extended assert macro

#ifndef __XASSERT_H
#define __XASSERT_H

/* Use this file if you want to have an extended assert macro
   with BugReport capability.
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef NDEBUG
  #define XASSERT(cond)                               ((void)0)
  #define XASSERT_T0(cond,t)                          ((void)0)
  #define XASSERT_T1(cond,t,a)                        ((void)0)
  #define XASSERT_T2(cond,t,a,b)                      ((void)0)
  #define XASSERT_T3(cond,t,a,b,c)                    ((void)0)
  #define XASSERT_T4(cond,t,a,b,c,d)                  ((void)0)
  #define XASSERT_T5(cond,t,a,b,c,d,e)                ((void)0)
  #define XASSERT_T6(cond,t,a,b,c,d,e,f)              ((void)0)
  #define XASSERT_T7(cond,t,a,b,c,d,e,f,g)            ((void)0)
  #define XASSERT_T8(cond,t,a,b,c,d,e,f,g,h)          ((void)0)
  #define XASSERT_T9(cond,t,a,b,c,d,e,f,g,h,i)        ((void)0)
  #define XASSERT_F(cond,fn,par)                      ((void)0)
  #define XASSERT_F0(cond,fn,par,t,a)                 ((void)0)
  #define XASSERT_F1(cond,fn,par,t,a)                 ((void)0)
  #define XASSERT_F2(cond,fn,par,t,a,b)               ((void)0)
  #define XASSERT_F3(cond,fn,par,t,a,b,c)             ((void)0)
  #define XASSERT_F4(cond,fn,par,t,a,b,c,d)           ((void)0)
  #define XASSERT_F5(cond,fn,par,t,a,b,c,d,e)         ((void)0)
  #define XASSERT_F6(cond,fn,par,t,a,b,c,d,e,f)       ((void)0)
  #define XASSERT_F7(cond,fn,par,t,a,b,c,d,e,f,g)     ((void)0)
  #define XASSERT_F8(cond,fn,par,t,a,b,c,d,e,f,g,h)   ((void)0)
  #define XASSERT_F9(cond,fn,par,t,a,b,c,d,e,f,g,h,i) ((void)0)
  #define XASSERT_ID
#else

  /* callback function on assertion failure */
  typedef void (*tXAssertFunc)(void*);

  /* assert ID for printing file versions */
  typedef struct _tXAssertID {
    char id[8];                   /* sanity check */
    struct _tXAssertID *self;     /* sanity check */
    char *file;                   /* name of file */
    char *date;                   /* compile date of file */
    char *time;                   /* compile time of file */
  } tXAssertID;

  extern void __x_assert(char*,char*,int,char*,char*,int,tXAssertFunc,void*,char*,...);

  #define XASSERT(cond)                               do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,0); } while(0)
  #define XASSERT_T0(cond,t)                          do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t); } while(0)
  #define XASSERT_T1(cond,t,a)                        do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a); } while(0)
  #define XASSERT_T2(cond,t,a,b)                      do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b); } while(0)
  #define XASSERT_T3(cond,t,a,b,c)                    do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e); } while(0)
  #define XASSERT_T4(cond,t,a,b,c,d)                  do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e); } while(0)
  #define XASSERT_T5(cond,t,a,b,c,d,e)                do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e); } while(0)
  #define XASSERT_T6(cond,t,a,b,c,d,e,f)              do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e,f); } while(0)
  #define XASSERT_T7(cond,t,a,b,c,d,e,f,g)            do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e,f,g); } while(0)
  #define XASSERT_T8(cond,t,a,b,c,d,e,f,g,h)          do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e,f,g,h); } while(0)
  #define XASSERT_T9(cond,t,a,b,c,d,e,f,g,h,i)        do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,0,0,t,a,b,c,d,e,f,g,h,i); } while(0)
  #define XASSERT_F(cond,fn,par)                      do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),0); } while(0)
  #define XASSERT_F0(cond,fn,par,t,a)                 do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t); } while(0)
  #define XASSERT_F1(cond,fn,par,t,a)                 do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a); } while(0)
  #define XASSERT_F2(cond,fn,par,t,a,b)               do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b); } while(0)
  #define XASSERT_F3(cond,fn,par,t,a,b,c)             do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c); } while(0)
  #define XASSERT_F4(cond,fn,par,t,a,b,c,d)           do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d); } while(0)
  #define XASSERT_F5(cond,fn,par,t,a,b,c,d,e)         do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d,e); } while(0)
  #define XASSERT_F6(cond,fn,par,t,a,b,c,d,e,f)       do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d,e,f); } while(0)
  #define XASSERT_F7(cond,fn,par,t,a,b,c,d,e,f,g)     do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d,e,f,g); } while(0)
  #define XASSERT_F8(cond,fn,par,t,a,b,c,d,e,f,g,h)   do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d,e,f,g,h); } while(0)
  #define XASSERT_F9(cond,fn,par,t,a,b,c,d,e,f,g,h,i) do { if (!(cond)) __x_assert(__DATE__,__TIME__,0,#cond,__FILE__,__LINE__,(tXAsssertFunc)(fn),(void*)(par),t,a,b,c,d,e,f,g,h,i); } while(0)
  #define XASSERT_ID static tXAssertID _XAssertID={ { '$','@','X','A','S','S','@','$' },&_XAssertID,__FILE__,__DATE__,__TIME__ };
#endif

#ifdef __cplusplus
};
#endif

#endif /* __XASSERT_H */
